package at.jku.ssw.pi.ue08;

import at.jku.ssw.pi.Out;
import at.jku.ssw.pi.dot.DotMaker;
import at.jku.ssw.pi.graph.Edge;
import at.jku.ssw.pi.graph.Graph;
import at.jku.ssw.pi.graph.Vertex;

public class WarshallTest {

	public static void main(String[] args) {
		testVO();
		test1();
		test2();
	}

	static void testVO() {
		Out.println("\n*** Test VO - transitive hull (directed, unweighted) ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		Vertex e = new Vertex('E');
		g.addVertex(e);

		g.addEdge(new Edge(a, b, Edge.Kind.Directed));
		g.addEdge(new Edge(b, c, Edge.Kind.Directed));
		g.addEdge(new Edge(c, a, Edge.Kind.Directed));
		g.addEdge(new Edge(d, a, Edge.Kind.Directed));
		g.addEdge(new Edge(e, d, Edge.Kind.Directed));
		makeDot("VOa.dot", g);
		Graph hull = Graphs.makeTransitiveHull(g);
		Out.println(String.format("Warshall: %d edges in graph, %d edges in transitive hull", g.edges.size(),
				hull.edges.size()));
		makeDot("VOb.dot", hull);
	}

	static void test1() {
		Out.println("\n*** Test 1 - transitive hull (directed, unweighted) ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		Vertex e = new Vertex('E');
		g.addVertex(e);
		Vertex f = new Vertex('F');
		g.addVertex(f);
		g.addEdge(new Edge(a, b, Edge.Kind.Directed));
		g.addEdge(new Edge(a, d, Edge.Kind.Directed));
		g.addEdge(new Edge(a, c, Edge.Kind.Directed));
		g.addEdge(new Edge(c, f, Edge.Kind.Directed));
		g.addEdge(new Edge(d, b, Edge.Kind.Directed));
		g.addEdge(new Edge(d, e, Edge.Kind.Directed));
		g.addEdge(new Edge(d, f, Edge.Kind.Directed));
		g.addEdge(new Edge(e, c, Edge.Kind.Directed));
		g.addEdge(new Edge(f, e, Edge.Kind.Directed));
		makeDot("1a.dot", g);
		Graph hull = Graphs.makeTransitiveHull(g);
		Out.println(String.format("Warshall: %d edges in graph, %d edges in transitive hull", g.edges.size(),
				hull.edges.size()));
		makeDot("1b.dot", hull);
	}

	static void test2() {
		Out.println("\n*** Test 2 - transitive hull (undirected, unweighted) ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		g.addEdge(new Edge(a, b, 0, Edge.Kind.Undirected));
		g.addEdge(new Edge(b, c, 0, Edge.Kind.Undirected));
		g.addEdge(new Edge(b, d, 0, Edge.Kind.Undirected));
		g.addEdge(new Edge(c, a, 0, Edge.Kind.Undirected));
		makeDot("2a.dot", g);
		Graph hull = Graphs.makeTransitiveHull(g);
		Out.println(String.format("Warshall: %d edges in graph, %d edges in transitive hull", g.edges.size(),
				hull.edges.size()));
		makeDot("2b.dot", hull);
	}

	public static void makeDot(String fileName, Graph g) {
		Out.open(fileName);
		Out.println(DotMaker.makeDotForGraph(g));
		Out.close();
	}
}
